# Example API

Browser and node module for making API requests against [Example API](http://example.com).

**Please note: This module uses [Popsicle](https://github.com/blakeembrey/popsicle) to make API requests. Promises must be supported or polyfilled on all target environments.**

## Installation

```
npm install example-api --save
bower install example-api --save
```

## Usage

### Node

```javascript
var ExampleApi = require('example-api');

var client = new ExampleApi();
```

### Browsers

```html
<script src="example-api/index.js"></script>

<script>
  var client = new ExampleApi();
</script>
```

### Options

You can set options when you initialize a client or at any time with the `options` property. You may also override options for a single request by passing an object as the second argument of any request method. For example:

```javascript
var client = new ExampleApi({ ... });

client.options = { ... };

client.resource('/').get(null, {
  baseUri: 'http://example.com',
  headers: {
    'Content-Type': 'application/json'
  }
});
```

#### Base URI

You can override the base URI by setting the `baseUri` property, or initializing a client with a base URI. For example:

```javascript
new ExampleApi({
  baseUri: 'https://example.com'
});
```

#### Base URI Parameters

If the base URI has parameters inline, you can set them by updating the `baseUriParameters` property. For example:

```javascript
client.options.baseUriParameters.version = 'v3';
```

### Resources

All methods return a HTTP request instance of [Popsicle](https://github.com/blakeembrey/popsicle), which allows the use of promises (and streaming in node).

#### resources.resource

```js
var resource = client.resources.resource;
```

##### OPTIONS

Some requests require authentication

```js
resource.options().then(function (res) { ... });
```

##### CONNECT

Some requests require authentication

```js
resource.connect().then(function (res) { ... });
```

##### TRACE

Some requests require authentication

```js
resource.trace().then(function (res) { ... });
```

##### PATCH

Some requests require authentication

```js
resource.patch().then(function (res) { ... });
```

##### DELETE

Some requests require authentication

```js
resource.delete().then(function (res) { ... });
```

##### PUT

Some requests require authentication

```js
resource.put().then(function (res) { ... });
```

##### GET

get the first one

```js
resource.get().then(function (res) { ... });
```

##### Headers

```javascript
resource.get(null, {
  headers: { ... }
});
```

* **x-custom** _string_

#### resources.resource.resourceId(resourceId)

* **resourceId** _string_

Which resoure would you like to view

This is a resource description *with* some _markdown_ embedded in it

```js
var resource = client.resources.resource.resourceId(resourceId);
```

##### GET

Instagram’s API uses the [OAuth 2.0 protocol](http://tools.ietf.org/html/draft-ietf-oauth-v2-12) for simple, but effective authentication and authorization. OAuth 2.0 is much easier to use than previous schemes; developers can start using the Instagram API almost immediately. The one thing to keep in mind is that all requests to the API must be made over SSL (https:// not http://)

## Do you need to authenticate?

For the most part, Instagram’s API only requires the use of a _client_id). A client_id simply associates your server, script, or program with a specific application. However, some requests require authentication - specifically requests made on behalf of a user. Authenticated requests require an _access_token_. These tokens are unique to a user and should be stored securely. Access tokens may expire at any time in the future.

Note that in many situations, you may not need to authenticate users at all. For instance, you may request popular photos without authenticating (i.e. you do not need to provide an access_token; just use your client ID with your request). We only require authentication in cases where your application is making requests on behalf of a user (commenting, liking, browsing a user’s feed, etc.).

## Receiving an access_token

```js
resource.get().then(function (res) { ... });
```

##### Query Parameters

```javascript
resource.get({ ... });
```

* **filter** _string_

What to filter

##### POST

```js
resource.post().then(function (res) { ... });
```

##### Body

**application/json**

**application/x-www-form-urlencoded**

{
  "name": {
    "description": "The name of the resource to create",
    "type": "string",
    "example": "Comment",
    "displayName": "name"
  },
  "description": {
    "description": "A description of the resource to create",
    "type": "string",
    "example": "User-generated content pertinent to the associated blog post",
    "displayName": "description"
  }
}

**multipart/form-data**

{
  "name": {
    "description": "The name of the resource to create",
    "type": "string",
    "example": "Comment",
    "displayName": "name"
  },
  "description": {
    "description": "A description of the resource to create",
    "type": "string",
    "example": "User-generated content pertinent to the associated blog post",
    "displayName": "description"
  }
}

#### resources.another.resource

This is a description. It can be long.

# Headers

It can have headers, and _italic_, and **bold** text.

# Length

Because it is arbitrary markdown, it can be arbitrarily long.

```js
var resource = client.resources.another.resource;
```

##### CONNECT

Some requests require authentication

```js
resource.connect().then(function (res) { ... });
```

##### HEAD

Some requests require authentication

```js
resource.head().then(function (res) { ... });
```

##### GET

Some requests require authentication

```js
resource.get().then(function (res) { ... });
```

##### Query Parameters

```javascript
resource.get({ ... });
```

* **page** _integer_

Which page to display

* **order** _string, one of (oldest, newest), default: newest_

The sort order of resources

* **query** _string_

A query parameter

#### resources.resource-with-headers

```js
var resource = client.resources.resource-with-headers;
```

##### GET

```js
resource.get().then(function (res) { ... });
```

##### Headers

```javascript
resource.get(null, {
  headers: { ... }
});
```

* **Custom header** _string_

This header is used to send data that...

* **Parameterized header** _string_

#### resources.secured-resource

```js
var resource = client.resources.secured-resource;
```

##### GET

```js
resource.get().then(function (res) { ... });
```

#### resources.resource-with-method-level-traits

```js
var resource = client.resources.resource-with-method-level-traits;
```

##### GET

get the first one

```js
resource.get().then(function (res) { ... });
```

#### resources.resource-with-form-and-multipart-form-parameters

```js
var resource = client.resources.resource-with-form-and-multipart-form-parameters;
```

##### GET

```js
resource.get().then(function (res) { ... });
```

##### Query Parameters

```javascript
resource.get({ ... });
```

* **Some Query Param** _string_

Your value for some thing.

##### Body

**application/json**

**application/x-www-form-urlencoded**

{
  "api_key": {
    "displayName": "API Key",
    "description": "Your license key for the application. Please contact developer@nzpost.co.nz for a license key",
    "type": "string",
    "required": true,
    "example": "c4f820f0420a013ea143230c290fbf99"
  }
}

**multipart/form-data**

{
  "api_key": {
    "displayName": "API Key",
    "description": "Your license key for the application. Please contact developer@nzpost.co.nz for a license key",
    "type": "string",
    "required": true,
    "example": "c4f820f0420a013ea143230c290fbf99"
  }
}

#### resources.resource-with-repeatable-params

```js
var resource = client.resources.resource-with-repeatable-params;
```

##### POST

```js
resource.post().then(function (res) { ... });
```

##### Query Parameters

```javascript
resource.post(null, { query: { ... } });
```

* **someParam** _string_

* **notRepeatable** _string_

##### Headers

```javascript
resource.post(null, {
  headers: { ... }
});
```

* **someHeader** _string_

* **x-meta-{*}** _string_

##### Body

**application/x-www-form-urlencoded**

{
  "someFormParam": {
    "repeat": true,
    "displayName": "someFormParam",
    "type": "string"
  }
}

**multipart/form-data**

{
  "someMultipartFormParam": {
    "type": "file",
    "repeat": true,
    "displayName": "someMultipartFormParam"
  },
  "someMultipartFormParamWithMultipleTypes": [
    {
      "type": "file",
      "displayName": "someMultipartFormParamWithMultipleTypes"
    },
    {
      "type": "string",
      "repeat": true,
      "displayName": "someMultipartFormParamWithMultipleTypes"
    }
  ]
}



### Custom Resources

You can make requests to a custom path in the API using the `#resource(path)` method.

```javascript
client.resource('/example/path').get();
```

## License

Apache 2.0
